@@ -1,6 +1,6 @@
-/* 
+/*
     SuperCat - The Super Cat Colorizer
-    Copyright (C) 2007-2017 - Thomas G. Anderson
+    Copyright (C) 2007-2020 - Mark P. Anderson, Thomas G. Anderson
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,16 +27,17 @@ struct spc_s {
     char   *htmrgb;  /* html color codes RGB */
     int     number;  /* 1-9, 0:all */
     size_t  htmlen;  /* length of htmrgb string */
-    char    aryidx;  /* array index 1-255, 0=default */
+    int     aryidx;  /* array index 1-255, 0=default */
     char    colour;  /* enum */
     char    attrib;  /* -, b, i, u */
-    char    patype;  /* c, s, r, R, t */
+    char    patype;  /* c, s, r, R, t, T, u */
 };
 
 spc_t Spc [NREGEX+1];   /* 0 is default */
-int   Spn = 1;
-char  Att;
-char  Col;
+int   Spn = 1;          /* number of above read from config file */
+int   Att;
+int   Col;
+FILE *Fp = 0;               /* output file pointer for -e and -E */
 
 int   opt_debug = 0;        /* multi-level */
 int   opt_reverse = 0;      /* 0:whi on blk, 1:blk on whi */
@@ -52,37 +53,76 @@ char *inp_file = 0;
 int   inp_lineno = 0;
 char *typ_suffix = 0;
 
+char *html_colour_lookup (int colour);
+int   colour_lookup (char *str);
+int   attrib_lookup (char  chr);
+int   number_lookup (char  chr);
+int   patype_lookup (char  chr);
+
 void
 help ()
 {
     puts ("usage: spc [OPTIONS] [FILE]");
+    puts ("           -e color,string|(RE)");
+    puts ("           -E color,attribute,number,type,string|(RE)");
     puts ("");
     puts ("  -c <CFG>, --config=CFG       specify non-standard spcrc config file");
     puts ("  -C <DIR>, --config-dir=DIR   specify non-standard config directory");
     puts ("  -d,       --debug            turn on debug information");
     puts ("  -D,       --no-extension     don't look at ./.spcrc[-ext] for config file");
+    puts ("  -e,       --cmd-s            config command - short");
+    puts ("  -E,       --cmd-l            config command - long");
     puts ("  -h,       --help             print this help and exit");
     puts ("  -m,       --matching         print only lines that have matches");
     puts ("  -n,       --line-numbers     precede each line with a line number");
     puts ("  -r,       --reverse          reverse black and white");
     puts ("  -R,       --nohomedir        don't look in ~/.spcrc for config files");
-    puts ("  -S,       --printsysdir      printf system directory and exit");
+    puts ("  -S,       --printsysdir      print system directory and exit");
     puts ("  -t <TYP>, --type=TYP         specify the effective file type extension");
     puts ("  -v,       --version          print version information and exit");
     puts ("  -w,       --webpage          generate web page");
     puts ("");
-    puts ("Report bugs to <bug-spc@nosredna.net>");
+    puts ("      color: color to display (COL)");
+    puts ("        blk : black     yel : yellow");
+    puts ("        blu : blue      mag : magenta");
+    puts ("        grn : green     red : red");
+    puts ("        cya : cyan      whi : white");
+    puts ("      attribute: display attributes (A)");
+    puts ("          n : normal      r : reverse");
+    puts ("          b : bold        k : blink");
+    puts ("          u : underline");
+    puts ("      number: number of occurrences per line, [0-9, 0=all]");
+    puts ("      type: string or regular-expression(RE) type (T)");
+    puts ("          c : characters");
+    puts ("          s : string");
+    puts ("          r : RE - case   sensitive");
+    puts ("          R : RE - case insensitive");
+    puts ("          t : RE for Unix time conversion : MMDDHHMMSS");
+    puts ("          T : RE for Unix time conversion : YYMMDDHHMM");
+    puts ("          u : RE for Unix time conversion :   HH:MM:SS");
+    puts ("          U : RE for Unix time conversion : YYYY:MM:DD");
+    puts ("");
+    puts ("  Configuration File Format");
+    puts ("");
+    puts ("#        1         2         3         4         5");
+    puts ("#2345678901234567890123456789012345678901234567890123456789");
+    puts ("# HTML-COLOR         COL A N T STRING-or-REGULAR-EXPRESSION");
+    puts ("#################### ### # # # ############################");
     exit (0);
 }
 
 void
 version ()
 {
-    printf ("spc (The SuperCat Colorizer) Version %s January 1, 2017", VERSION);
+    printf ("spc (SuperCat Colorizer) Version %s January 1, 2020\n", VERSION);
     puts   ("");
-    puts   ("Copyright (C) 2007-2015 - Thomas G. Anderson");
-    puts   ("This is free software; see the source for copying conditions.  There is NO");
-    puts   ("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.");
+    puts   ("Copyright (C) 2007-2020 - Mark P. Anderson, Thomas G. Anderson");
+    puts   ("");
+    puts   ("This is free software, see the source for copying conditions.");
+    puts   ("There is NO warranty not even for MERCHANTABILITY or FITNESS");
+    puts   ("FOR A PARTICULAR PURPOSE.  Released under GPLv3.");
+    puts   ("");
+    puts   ("Report bugs or suggestions to <bug-spc @ nosredna.net>");
     exit   (0);
 }
 
@@ -96,40 +136,117 @@ sysdir ()
 void
 err_doit (int errnoflag, int error, const char *fmt, va_list ap)
 {
-	char buf [BUFSIZ];
+    char buf [BUFSIZ];
 
-	vsnprintf (buf, BUFSIZ, fmt, ap);
-	if (errnoflag)
-		snprintf (buf + strlen(buf), BUFSIZ - strlen(buf), ": %s",
-		strerror (error));
-	strcat (buf, "\n");
-	fflush (stdout);
-	fputs  (buf, stderr);
-	fflush (NULL);
+    vsnprintf (buf, BUFSIZ, fmt, ap);
+    if (errnoflag)
+        snprintf (buf + strlen(buf), BUFSIZ - strlen(buf), ": %s", strerror (error));
+    strcat (buf, "\n");
+    fflush (stdout);
+    fputs  (buf, stderr);
+    fflush (NULL);
 }
 
 void
 err_quit (const char *fmt, ...)
 {
-	va_list ap;
+    va_list ap;
 
-	va_start (ap, fmt);
-	err_doit (0, 0, fmt, ap);
-	va_end   (ap);
-	exit (-1);
+    va_start (ap, fmt);
+    err_doit (0, 0, fmt, ap);
+    va_end   (ap);
+    exit (-1);
 }
 
 void
 err_sys (const char *fmt, ...)
 {
-	va_list ap;
+    va_list ap;
 
-	va_start (ap, fmt);
-	err_doit (1, errno, fmt, ap);
-	va_end   (ap);
-	exit (-1);
+    va_start (ap, fmt);
+    err_doit (1, errno, fmt, ap);
+    va_end   (ap);
+    exit (-1);
 }
-#ifdef HAVE_GETOPT_H
+
+/* -e color,string|(RE) */
+void
+option_e (char *str, char *cmd)
+{
+    char *s, *q;
+
+    if (!Fp) Fp = tmpfile ();
+
+    if ( (strlen (str) < 5) || (str [3] != ',') ) {
+        printf ("%s invalid option string, format is \"color,string|(RE)\"\n", cmd);
+        exit (-1);
+    }
+    str [3] = 0;  /* null terminate color */
+
+    /* if there is no enclosing (...) add them */
+    s = &str [4];
+    if (!strchr (s, '(')) {
+        q = malloc (strlen (s)+3);
+        strcpy (q, "(");
+        strcat (q, s);
+        strcat (q, ")");
+        s = q;
+    }
+
+    if (colour_lookup (&str [0]) == -1) {
+        printf ("%s invalid option string color (%s), use: blk|blu|grn|cya|yel|mag|red|whi\n", cmd, &str [0]);
+        exit (-1);
+    }
+
+    /* add this rule to the temporary config file in /tmp */
+//   printf (    "%-20s %3s %c %d %c %s\n", html_colour_lookup (colour_lookup (&str [0])), &str [0], '-', 1, 'r', s);
+    fprintf (Fp, "%-20s %3s %c %d %c %s\n", html_colour_lookup (colour_lookup (&str [0])), &str [0], '-', 1, 'r', s);
+}
+
+/* -E color,att,num,typ,string|(RE) */
+/* -E ccc,a,n,t,string|(RE) */
+/* -E 0123456789string|(RE) */
+void
+option_E (char *str, char *cmd)
+{
+    if (!Fp) Fp = tmpfile ();
+
+    if ( (strlen (str) < 11) || (str [3] != ',') || (str [5] != ',') || (str [7] != ',' || (str [9] != ',') ) ) {
+        printf ("%s invalid option string, format is \"color,att,num,typ,string|(RE)\"\n", cmd);
+        exit (-1);
+    }
+    str [3] = 0;
+    str [5] = 0;
+    str [7] = 0;
+    str [9] = 0;
+
+    if (colour_lookup (&str [0]) == -1) {
+        printf ("%s invalid option string color (%s), use: blk|blu|grn|cya|yel|mag|red|whi\n", cmd, &str [0]);
+        exit (-1);
+    }
+    if (attrib_lookup (str [4]) == -1) {
+        printf ("%s invalid option string attribute (%c), use: -|n|b|r|u|k\n", cmd, str [4]);
+        exit (-1);
+    }
+    if (number_lookup (str [6]) == -1) {
+        printf ("%s invalid option string number (%c), use: 0-9\n", cmd, str [6]);
+        exit (-1);
+    }
+    if (patype_lookup (str [8]) == -1) {
+        printf ("%s invalid option string type (%c), use: c|s|r|R|t|T|u|U\n", cmd, str [8]);
+        exit (-1);
+    }
+
+    /* add this rule to the temporary config file in /tmp */
+//   printf (    "%-20s %3s %c %c %c %s\n", html_colour_lookup (colour_lookup (&str [0])), &str [0], str [4], str [6], str [8], &str [10]);
+    fprintf (Fp, "%-20s %3s %c %c %c %s\n", html_colour_lookup (colour_lookup (&str [0])), &str [0], str [4], str [6], str [8], &str [10]);
+}
+
+/* #ifdef HAVE_GETOPT_H */
+/* getops cores on --config-cmd- so why use it? */
+
+#if 0
+
 void
 options (int ac, char **av)
 {
@@ -140,6 +257,8 @@ options (int ac, char **av)
         {"config-dir",    1, 0, 'C' },  /* non-standard config directory */
         {"debug",         0, 0, 'd' },  /* turn on debug */
         {"no-extension",  0, 0, 'D' },  /* don't do extension scraping */
+        {"cmd-s",         1, 0, 'e' },  /* command line configuration command */
+        {"cmd-l",         1, 0, 'E' },  /* command line configuration command */
         {"help",          0, 0, 'h' },  /* helpful information and exit */
         {"matching",      0, 0, 'm' },  /* only show matching portions of a line */
         {"line-numbers",  0, 0, 'n' },  /* pre-pend line numbers */
@@ -151,11 +270,10 @@ options (int ac, char **av)
         {"webpage",       0, 0, 'w' }   /* generate html output */
     };
 
-
     while (1) {
         int opt_idx = 0;
 
-        switch ((c = getopt_long (ac, av, "c:C:dDhmnrRSt:vw", opt_tbl, &opt_idx))) {
+        switch ((c = getopt_long (ac, av, "c:C:dDe:E:hmnrRSt:vw", opt_tbl, &opt_idx))) {
           case -1:
             goto DONE;
 
@@ -173,7 +291,7 @@ options (int ac, char **av)
           case 'n': opt_line_number = 1; break;
           case 'r': opt_reverse = 1; break;
           case 'R': opt_nohomerc = 1; break;
-          case 'S': sysdir (); break; 
+          case 'S': sysdir (); break;
           case 'v': version (); break;
           case 'w': opt_webpage = 1; break;
 
@@ -192,6 +310,14 @@ options (int ac, char **av)
             strcpy (opt_type, optarg);
             break;
 
+          case 'e':
+            option_e (optarg);
+            break;
+
+          case 'E':
+            option_E (optarg);
+            break;
+
           case '?':
             break;
 
@@ -207,13 +333,16 @@ DONE:
         strcpy (inp_file, av [optind]);
     }
 }
+
 #else
 
+/* without getopt */
+
 void
 options (int ac, char **av)
 {
     int   i;
-    char *p;
+    char *p, *q;
 
     for (i=1; i<ac; i++) {
         if (!strcmp (av [i], "-h") || !strcmp (av [i], "--help"))           help ();
@@ -228,58 +357,127 @@ options (int ac, char **av)
         if (!strcmp (av [i], "-w") || !strcmp (av [i], "--webpage"))      { opt_webpage = 1;     continue; }
 
         if (!strcmp (av [i], "-c")) {
-            if (!av[++i]) err_quit ("missing parameter to -c");
-            opt_config = malloc (strlen (av[i])+1);
+            if (!av [++i]) err_quit ("-c missing parameter");
+            opt_config = malloc (strlen (av [i])+1);
             strcpy (opt_config, av [i]);
             continue;
         }
-        if (!strncmp (av [i], "--config", strlen ("--config"))) {
-            if (*(p = av [i] + strlen ("--config")) != '=')
-                err_quit ("invalid character in --config");
-            if (!*(++p))
-                err_quit ("missing parameter to --config");
-            opt_config = malloc (strlen (p)+1);
-            strcpy (opt_config, p);
-            continue;
-        }
         if (!strcmp (av [i], "-C")) {
-            if (!av[++i]) err_quit ("missing parameter to -C");
-            opt_directory = malloc (strlen (av[i])+1);
+            if (!av [++i]) err_quit ("-C missing parameter");
+            opt_directory = malloc (strlen (av [i])+1);
             strcpy (opt_directory, av [i]);
             continue;
         }
+
+        if (!strcmp (av [i], "-e")) {
+            if (!av [i+1]) err_quit ("-e missing parameter\n");
+            option_e (av [i+1], av [i]);
+            i++;
+            continue;
+        }
+        if (!strcmp (av [i], "-E")) {
+            if (!av [i+1]) err_quit ("-E missing parameter\n");
+            option_E (av [i+1], av [i]);
+            i++;
+            continue;
+        }
+
+        if (!strncmp (av [i], "--cmd-s", strlen ("--cmd-s"))) {
+            p = av [i] + strlen ("--cmd-s");
+            if (*p == '=')
+                *p++ = 0;
+            else
+                p = av [++i];
+
+            if (!p || !*p) err_quit ("--cmd-s missing parameter\n");
+
+            option_e (p, av [i]);
+            continue;
+        }
+
+        if (!strncmp (av [i], "--cmd-l", strlen ("--cmd-l"))) {
+            p = av [i] + strlen ("--cmd-l");
+            if (*p == '=')
+                *p++ = 0;
+            else
+                p = av [++i];
+
+            if (!p || !*p) err_quit ("--cmd-l missing parameter\n");
+
+            option_E (p, av [i]);
+            continue;
+        }
+
         if (!strncmp (av [i], "--config-dir", strlen ("--config-dir"))) {
-            if (*(p = av [i] + strlen ("--config-dir")) != '=')
-                err_quit ("invalid character in --config-dir");
-            if (!*(++p))
-                err_quit ("missing parameter to --config-dir");
+            p = av [i] + strlen ("--config-dir");
+            if (*p == '=')
+                *p++ = 0;
+            else
+                p = av [++i];
+
+            if (!p || !*p) err_quit ("--config-dir missing parameter");
+
             opt_directory = malloc (strlen (p)+1);
             strcpy (opt_directory, p);
             continue;
         }
+
+        if (!strncmp (av [i], "--config", strlen ("--config"))) {
+            p = av [i] + strlen ("--config");
+            if (*p == '=')
+                *p++ = 0;
+            else {
+                if (strcmp (av [i], "--config")) {
+                    if ((p = strchr (av [i], '='))) *p = 0;
+                    printf ("%s unknown option\n", av [i]);
+                    exit (-1);
+                }
+                p = av [++i];
+            }
+
+            if (!p || !*p) err_quit ("--config missing parameter");
+
+            opt_config = malloc (strlen (p)+1);
+            strcpy (opt_config, p);
+            continue;
+        }
+
         if (!strcmp (av [i], "-t")) {
-            if (!av[++i]) err_quit ("missing parameter to -t");
-            opt_type = malloc (strlen (av[i])+1);
+            if (!av [++i]) err_quit ("-t missing parameter");
+            opt_type = malloc (strlen (av [i])+1);
             strcpy (opt_type, av [i]);
             continue;
         }
+
         if (!strncmp (av [i], "--type", strlen ("--type"))) {
-            if (*(p = av [i] + strlen ("--type")) != '=')
-                err_quit ("invalid character in --type");
-            if (!*(++p))
-                err_quit ("missing parameter to --type");
+            p = av [i] + strlen ("--type");
+            if (*p == '=')
+                *p++ = 0;
+            else
+                p = av [++i];
+
+            if (!p || !*p) err_quit ("--type missing parameter");
+
             opt_type = malloc (strlen (p)+1);
             strcpy (opt_type, p);
             continue;
         }
+
+        if (av [i] [0] == '-') {
+            if ((p = strchr (av [i], '='))) *p = 0;
+            printf ("%s unknown option\n", av [i]);
+            exit (-1);
+        }
+
         if (i == (ac-1)) {
             inp_file = malloc (strlen (av [i])+1);
             strcpy (inp_file, av [i]);
             continue;
         }
-        err_quit ("unknown option %s", av [i]);
+        err_quit ("%s unknown option", av [i]);
     }
 }
+
 #endif
 
 void
@@ -384,6 +582,8 @@ configure ()
     FILE    *rc;
     char    *home, *temp;
 
+    if (Fp) { rewind (Fp); return (Fp); }  /* temporary file built from command line arguments */
+
     /*
      *  Open *conf_file, ./.spcrc, ~/.spcrc, /etc/spcrc
      */
@@ -426,7 +626,24 @@ configure ()
     err_quit ("can't find a config file");
 }
 
-char
+char *
+html_colour_lookup (int colour)
+{
+    switch (colour) {
+      case BLK: return ("Black");
+      case RED: return ("Red");
+      case GRN: return ("Green");
+      case YEL: return ("Yellow");
+      case BLU: return ("Blue");
+      case MAG: return ("Magenta");
+      case CYA: return ("Cyan");
+      case WHI: return ("Black");
+       default: return ("White");
+    }
+    return ("White");
+}
+
+int
 colour_lookup (char *str)
 {
     if (!strcmp (str, "blk")) return (BLK);
@@ -437,7 +654,17 @@ colour_lookup (char *str)
     if (!strcmp (str, "mag")) return (MAG);
     if (!strcmp (str, "cya")) return (CYA);
     if (!strcmp (str, "whi")) return (WHI);
-    err_quit ("unknown color %s at line number %d", str, inp_lineno);
+    return (-1);
+}
+int
+colour_lookup_or_exit (char *str)
+{
+    int col = colour_lookup (str);
+
+    if (col == -1)
+      err_quit ("unknown color %s at line number %d", str, inp_lineno);
+
+    return (col);
 }
 
 int
@@ -454,22 +681,43 @@ number_lookup (char chr)
     if (chr == '7') return (7);
     if (chr == '8') return (8);
     if (chr == '9') return (9);
-    err_quit ("unknown number %c at line number %d", chr, inp_lineno);
+    return (-1);
+}
+int
+number_lookup_or_exit (char chr)
+{
+    int num = number_lookup (chr);
+
+    if (num == -1)
+      err_quit ("unknown number %c at line number %d", chr, inp_lineno);
+
+    return (num);
 }
 
-char
+int
 attrib_lookup (char chr)
 {
-    if (chr == '-') return ('0');
     if (chr == ' ') return ('0');
+    if (chr == '-') return ('0');
+    if (chr == 'n') return ('0');
     if (chr == 'b') return ('1');
     if (chr == 'u') return ('4');
     if (chr == 'k') return ('5');
     if (chr == 'r') return ('7');
-    err_quit ("unknown attribute %c at line number %d", chr, inp_lineno);
+    return (-1);
+}
+int
+attrib_lookup_or_exit (char chr)
+{
+    int att = attrib_lookup (chr);
+
+    if (att == -1)
+      err_quit ("unknown attribute %c at line number %d", chr, inp_lineno);
+
+    return (att);
 }
 
-char
+int
 patype_lookup (char chr)
 {
     if (chr == ' ') return (PATYPE_REGEXP);
@@ -477,8 +725,21 @@ patype_lookup (char chr)
     if (chr == 'r') return (PATYPE_REGEXP);
     if (chr == 'R') return (PATYPE_RECASE);
     if (chr == 's') return (PATYPE_STRING);
-    if (chr == 't') return (PATYPE_UNIXTM);
-    err_quit ("unknown pattern type %c at line number %d", chr, inp_lineno);
+    if (chr == 't') return (PATYPE_UNIXTM1);
+    if (chr == 'T') return (PATYPE_UNIXTM2);
+    if (chr == 'u') return (PATYPE_UNIXTM3);
+    if (chr == 'U') return (PATYPE_UNIXTM4);
+    return (-1);
+}
+int
+patype_lookup_or_exit (char chr)
+{
+    int typ = patype_lookup (chr);
+
+    if (typ == -1)
+      err_quit ("unknown pattern type %c at line number %d", chr, inp_lineno);
+
+    return (typ);
 }
 
 int
@@ -573,10 +834,15 @@ colour_rex (spc_t *spc, char *string, char *aryidx, int n)
 
             /*
              *  This hack converts a 10-digit Unix time-since-epoch
-             *  into a human-readable format MMDDHHMMSS within the
-             *  same 10 characters so the string stays the same size.
+             *  into a human-readable formats within the same 10
+             *  characters so the string stays the same size.
+             *
+             *  UNIXTM1 : t : MMDDHHMMSS
+             *  UNIXTM2 : T : YYMMDDHHMM
+             *  UNIXTM3 : u :   HH:MM:SS
+             *  UNIXTM4 : U : YYYY:MM:DD
              */
-            if (spc->patype == PATYPE_UNIXTM) {
+            if (spc->patype == PATYPE_UNIXTM1) {
                 if ((m->rm_eo - m->rm_so) != 10) continue;
                 unixtm = 0;
                 for (i=m->rm_so+j; i<m->rm_eo+j; i++) {
@@ -597,6 +863,72 @@ colour_rex (spc_t *spc, char *string, char *aryidx, int n)
                 string [i++] = (tm->tm_sec  / 10) + '0' ;
                 string [i++] = (tm->tm_sec  % 10) + '0' ;
             }
+            else if (spc->patype == PATYPE_UNIXTM2) {
+                if ((m->rm_eo - m->rm_so) != 10) continue;
+                unixtm = 0;
+                for (i=m->rm_so+j; i<m->rm_eo+j; i++) {
+                    unixtm *= 10;
+                    unixtm += string [i] - '0';
+                }
+                tm = localtime (&unixtm);
+                i = m->rm_so;
+                tm->tm_mon++;
+                tm->tm_year %= 100;
+                string [i++] = (tm->tm_year / 10) + '0' ;
+                string [i++] = (tm->tm_year % 10) + '0' ;
+                string [i++] = (tm->tm_mon  / 10) + '0' ;
+                string [i++] = (tm->tm_mon  % 10) + '0' ;
+                string [i++] = (tm->tm_mday / 10) + '0' ;
+                string [i++] = (tm->tm_mday % 10) + '0' ;
+                string [i++] = (tm->tm_hour / 10) + '0' ;
+                string [i++] = (tm->tm_hour % 10) + '0' ;
+                string [i++] = (tm->tm_min  / 10) + '0' ;
+                string [i++] = (tm->tm_min  % 10) + '0' ;
+            }
+            else if (spc->patype == PATYPE_UNIXTM3) {
+                if ((m->rm_eo - m->rm_so) != 10) continue;
+                unixtm = 0;
+                for (i=m->rm_so+j; i<m->rm_eo+j; i++) {
+                    unixtm *= 10;
+                    unixtm += string [i] - '0';
+                }
+                tm = localtime (&unixtm);
+                i = m->rm_so;
+                tm->tm_mon++;
+                string [i++] = ' ';
+                string [i++] = ' ';
+                string [i++] = (tm->tm_hour / 10) + '0' ;
+                string [i++] = (tm->tm_hour % 10) + '0' ;
+                string [i++] = ':';
+                string [i++] = (tm->tm_min  / 10) + '0' ;
+                string [i++] = (tm->tm_min  % 10) + '0' ;
+                string [i++] = ':';
+                string [i++] = (tm->tm_sec  / 10) + '0' ;
+                string [i++] = (tm->tm_sec  % 10) + '0' ;
+            }
+            else if (spc->patype == PATYPE_UNIXTM4) {
+                if ((m->rm_eo - m->rm_so) != 10) continue;
+                unixtm = 0;
+                for (i=m->rm_so+j; i<m->rm_eo+j; i++) {
+                    unixtm *= 10;
+                    unixtm += string [i] - '0';
+                }
+                tm = localtime (&unixtm);
+                i = m->rm_so;
+                tm->tm_mon++;
+                tm->tm_year += 1900;  /* four digits */
+                string [i++] = (tm->tm_year / 1000) + '0' ;
+                string [i++] =((tm->tm_year /  100) % 10) + '0' ;
+                tm->tm_year  = (tm->tm_year - 1900) % 100;
+                string [i++] = (tm->tm_year / 10) + '0' ;
+                string [i++] = (tm->tm_year % 10) + '0' ;
+                string [i++] = ':';
+                string [i++] = (tm->tm_mon  / 10) + '0' ;
+                string [i++] = (tm->tm_mon  % 10) + '0' ;
+                string [i++] = ':';
+                string [i++] = (tm->tm_mday / 10) + '0' ;
+                string [i++] = (tm->tm_mday % 10) + '0' ;
+            }
             if (opt_debug > 1) {
                 for (q=string, i=0; *q; q++) {
                     if (i++ == m->rm_so) putchar ('<');
@@ -629,16 +961,28 @@ colour_sca (char *string, char *aryidx, int n)
      *  in conjunction with 'opt_matching'.
      */
     for (spc=Spc, r=0; r<Spn; r++, spc++) {
-
-        if (spc->patype == PATYPE_REGEXP || spc->patype == PATYPE_RECASE)
+        switch (spc->patype) {
+          case PATYPE_REGEXP:
+          case PATYPE_RECASE:
+          case PATYPE_UNIXTM1:
+          case PATYPE_UNIXTM2:
+          case PATYPE_UNIXTM3:
+          case PATYPE_UNIXTM4:
             rtn |= colour_rex (spc, string, aryidx, n);
-        else if (spc->patype == PATYPE_STRING)
+            break;
+
+          case PATYPE_STRING:
             rtn |= colour_str (spc, string, aryidx, n);
-        else if (spc->patype == PATYPE_CHARS)
-            rtn |= colour_chr (spc, string, aryidx, n);
-        else    /* PATYPE_UNIXTM */
-            rtn |= colour_rex (spc, string, aryidx, n);
+            break;
 
+          case PATYPE_CHARS:
+            rtn |= colour_chr (spc, string, aryidx, n);
+            break;
+
+          default:
+            rtn |= colour_rex (spc, string, aryidx, n);
+            break;
+        }
     }
 
     if (opt_debug > 3) {
@@ -793,17 +1137,20 @@ create_rex (FILE *fp)
             *p = 0;
         }
         spc->aryidx = Spn;
-        spc->colour = colour_lookup (&buffer [COLUMN_COLOUR]);
-        spc->attrib = attrib_lookup ( buffer [COLUMN_ATTRIB]);
-        spc->number = number_lookup ( buffer [COLUMN_NUMBER]);
-        spc->patype = patype_lookup ( buffer [COLUMN_PATYPE]);
-        
+        spc->colour = colour_lookup_or_exit (&buffer [COLUMN_COLOUR]);
+        spc->attrib = attrib_lookup_or_exit ( buffer [COLUMN_ATTRIB]);
+        spc->number = number_lookup_or_exit ( buffer [COLUMN_NUMBER]);
+        spc->patype = patype_lookup_or_exit ( buffer [COLUMN_PATYPE]);
+
         spc->htmrgb = malloc (strlen (&buffer [COLUMN_HTMRGB]) + 1);
         strcpy (spc->htmrgb,&buffer [COLUMN_HTMRGB]);
         spc->htmlen = strlen (spc->htmrgb);
 
-        if (spc->patype == PATYPE_REGEXP ||
-            spc->patype == PATYPE_UNIXTM) {
+        if (spc->patype == PATYPE_REGEXP  ||
+            spc->patype == PATYPE_UNIXTM1 ||
+            spc->patype == PATYPE_UNIXTM2 ||
+            spc->patype == PATYPE_UNIXTM3 ||
+            spc->patype == PATYPE_UNIXTM4) {
             /*
              *  Compile the regular expression
              */
