@@ -1,6 +1,6 @@
 /* 
     SuperCat - The Super Cat Colorizer
-    Copyright (C) 2007-2015 - Thomas G. Anderson
+    Copyright (C) 2007-2020 - Mark P. Anderson, Thomas G. Anderson
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@
 # include <string.h>
 #endif
 
+#undef HAVE_GETOPT_H
 #if HAVE_GETOPT_H
 # include <getopt.h>
 #endif
@@ -62,13 +63,16 @@
 
 #define NMATCH  16
 #define NREGEX  16
-#define BUFFER  1024
+#define BUFFER  4096
 
 #define PATYPE_CHARS    'c'
 #define PATYPE_STRING   's'
 #define PATYPE_REGEXP   'r'	// case   sensitive RE
 #define PATYPE_RECASE   'R'	// case insensitive RE
-#define PATYPE_UNIXTM   't'
+#define PATYPE_UNIXTM1  't'
+#define PATYPE_UNIXTM2  'T'
+#define PATYPE_UNIXTM3  'u'
+#define PATYPE_UNIXTM4  'U'
 
 #define COLUMN_HTMRGB  0
 #define COLUMN_COLOUR 21 
